#!/usr/bin/python
from __future__ import absolute_import, division, print_function
# Copyright 2019-2020 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fmgr_wanprof_system_virtualwanlink
short_description: Configure redundant internet connections using SD-WAN (formerly virtual WAN link).
description:
    - This module is able to configure a FortiManager device.
    - Examples include all parameters and values which need to be adjusted to data sources before usage.

version_added: "2.10"
author:
    - Link Zheng (@chillancezen)
    - Jie Xue (@JieX19)
    - Frank Shen (@fshen01)
    - Hongbin Lu (@fgtdev-hblu)
notes:
    - Running in workspace locking mode is supported in this FortiManager module, the top
      level parameters workspace_locking_adom and workspace_locking_timeout help do the work.
    - To create or update an object, use state present directive.
    - To delete an object, use state absent directive.
    - Normally, running one module can fail when a non-zero rc is returned. you can also override
      the conditions to fail or succeed with parameters rc_failed and rc_succeeded

options:
    bypass_validation:
        description: only set to True when module schema diffs with FortiManager API structure, module continues to execute without validating parameters
        required: false
        type: bool
        default: false
    workspace_locking_adom:
        description: the adom to lock for FortiManager running in workspace mode, the value can be global and others including root
        required: false
        type: str
    workspace_locking_timeout:
        description: the maximum time in seconds to wait for other user to release the workspace lock
        required: false
        type: int
        default: 300
    state:
        description: the directive to create, update or delete an object
        type: str
        required: true
        choices:
          - present
          - absent
    rc_succeeded:
        description: the rc codes list with which the conditions to succeed will be overriden
        type: list
        required: false
    rc_failed:
        description: the rc codes list with which the conditions to fail will be overriden
        type: list
        required: false
    adom:
        description: the parameter (adom) in requested url
        type: str
        required: true
    wanprof:
        description: the parameter (wanprof) in requested url
        type: str
        required: true
    wanprof_system_virtualwanlink:
        description: the top level parameters set
        required: false
        type: dict
        suboptions:
            fail-detect:
                type: str
                description: 'Enable/disable SD-WAN Internet connection status checking (failure detection).'
                choices:
                    - 'disable'
                    - 'enable'
            health-check:
                description: no description
                type: list
                suboptions:
                    _dynamic-server:
                        type: str
                        description: no description
                    addr-mode:
                        type: str
                        description: 'Address mode (IPv4 or IPv6).'
                        choices:
                            - 'ipv4'
                            - 'ipv6'
                    failtime:
                        type: int
                        description: 'Number of failures before server is considered lost (1 - 3600, default = 5).'
                    http-agent:
                        type: str
                        description: 'String in the http-agent field in the HTTP header.'
                    http-get:
                        type: str
                        description: 'URL used to communicate with the server if the protocol if the protocol is HTTP.'
                    http-match:
                        type: str
                        description: 'Response string expected from the server if the protocol is HTTP.'
                    interval:
                        type: int
                        description: 'Status check interval, or the time between attempting to connect to the server (1 - 3600 sec, default = 5).'
                    members:
                        type: str
                        description: 'Member sequence number list.'
                    name:
                        type: str
                        description: 'Status check or health check name.'
                    packet-size:
                        type: int
                        description: 'Packet size of a twamp test session,'
                    password:
                        description: no description
                        type: str
                    port:
                        type: int
                        description: 'Port number used to communicate with the server over the selected protocol.'
                    protocol:
                        type: str
                        description: 'Protocol used to determine if the FortiGate can communicate with the server.'
                        choices:
                            - 'ping'
                            - 'tcp-echo'
                            - 'udp-echo'
                            - 'http'
                            - 'twamp'
                            - 'ping6'
                    recoverytime:
                        type: int
                        description: 'Number of successful responses received before server is considered recovered (1 - 3600, default = 5).'
                    security-mode:
                        type: str
                        description: 'Twamp controller security mode.'
                        choices:
                            - 'none'
                            - 'authentication'
                    server:
                        description: no description
                        type: str
                    sla:
                        description: no description
                        type: list
                        suboptions:
                            id:
                                type: int
                                description: 'SLA ID.'
                            jitter-threshold:
                                type: int
                                description: 'Jitter for SLA to make decision in milliseconds. (0 - 10000000, default = 5).'
                            latency-threshold:
                                type: int
                                description: 'Latency for SLA to make decision in milliseconds. (0 - 10000000, default = 5).'
                            link-cost-factor:
                                description: no description
                                type: list
                                choices:
                                 - latency
                                 - jitter
                                 - packet-loss
                            packetloss-threshold:
                                type: int
                                description: 'Packet loss for SLA to make decision in percentage. (0 - 100, default = 0).'
                    threshold-alert-jitter:
                        type: int
                        description: 'Alert threshold for jitter (ms, default = 0).'
                    threshold-alert-latency:
                        type: int
                        description: 'Alert threshold for latency (ms, default = 0).'
                    threshold-alert-packetloss:
                        type: int
                        description: 'Alert threshold for packet loss (percentage, default = 0).'
                    threshold-warning-jitter:
                        type: int
                        description: 'Warning threshold for jitter (ms, default = 0).'
                    threshold-warning-latency:
                        type: int
                        description: 'Warning threshold for latency (ms, default = 0).'
                    threshold-warning-packetloss:
                        type: int
                        description: 'Warning threshold for packet loss (percentage, default = 0).'
                    update-cascade-interface:
                        type: str
                        description: 'Enable/disable update cascade interface.'
                        choices:
                            - 'disable'
                            - 'enable'
                    update-static-route:
                        type: str
                        description: 'Enable/disable updating the static route.'
                        choices:
                            - 'disable'
                            - 'enable'
            load-balance-mode:
                type: str
                description: 'Algorithm or mode to use for load balancing Internet traffic to SD-WAN members.'
                choices:
                    - 'source-ip-based'
                    - 'weight-based'
                    - 'usage-based'
                    - 'source-dest-ip-based'
                    - 'measured-volume-based'
            members:
                description: no description
                type: list
                suboptions:
                    _dynamic-member:
                        type: str
                        description: no description
                    comment:
                        type: str
                        description: 'Comments.'
                    gateway:
                        type: str
                        description: 'The default gateway for this interface. Usually the default gateway of the Internet service provider that this interfa...'
                    gateway6:
                        type: str
                        description: 'IPv6 gateway.'
                    ingress-spillover-threshold:
                        type: int
                        description: 'Ingress spillover threshold for this interface (0 - 16776000 kbit/s). When this traffic volume threshold is reached, n...'
                    interface:
                        type: str
                        description: 'Interface name.'
                    priority:
                        type: int
                        description: 'Priority of the interface (0 - 4294967295). Used for SD-WAN rules or priority rules.'
                    seq-num:
                        type: int
                        description: 'Sequence number(1-255).'
                    source:
                        type: str
                        description: 'Source IP address used in the health-check packet to the server.'
                    source6:
                        type: str
                        description: 'Source IPv6 address used in the health-check packet to the server.'
                    spillover-threshold:
                        type: int
                        description: 'Egress spillover threshold for this interface (0 - 16776000 kbit/s). When this traffic volume threshold is reached, ne...'
                    status:
                        type: str
                        description: 'Enable/disable this interface in the SD-WAN.'
                        choices:
                            - 'disable'
                            - 'enable'
                    volume-ratio:
                        type: int
                        description: 'Measured volume ratio (this value / sum of all values = percentage of link volume, 0 - 255).'
                    weight:
                        type: int
                        description: 'Weight of this interface for weighted load balancing. (0 - 255) More traffic is directed to interfaces with higher wei...'
            service:
                description: no description
                type: list
                suboptions:
                    addr-mode:
                        type: str
                        description: 'Address mode (IPv4 or IPv6).'
                        choices:
                            - 'ipv4'
                            - 'ipv6'
                    bandwidth-weight:
                        type: int
                        description: 'Coefficient of reciprocal of available bidirectional bandwidth in the formula of custom-profile-1.'
                    default:
                        type: str
                        description: 'Enable/disable use of SD-WAN as default service.'
                        choices:
                            - 'disable'
                            - 'enable'
                    dscp-forward:
                        type: str
                        description: 'Enable/disable forward traffic DSCP tag.'
                        choices:
                            - 'disable'
                            - 'enable'
                    dscp-forward-tag:
                        type: str
                        description: 'Forward traffic DSCP tag.'
                    dscp-reverse:
                        type: str
                        description: 'Enable/disable reverse traffic DSCP tag.'
                        choices:
                            - 'disable'
                            - 'enable'
                    dscp-reverse-tag:
                        type: str
                        description: 'Reverse traffic DSCP tag.'
                    dst:
                        type: str
                        description: 'Destination address name.'
                    dst-negate:
                        type: str
                        description: 'Enable/disable negation of destination address match.'
                        choices:
                            - 'disable'
                            - 'enable'
                    dst6:
                        type: str
                        description: 'Destination address6 name.'
                    end-port:
                        type: int
                        description: 'End destination port number.'
                    gateway:
                        type: str
                        description: 'Enable/disable SD-WAN service gateway.'
                        choices:
                            - 'disable'
                            - 'enable'
                    groups:
                        type: str
                        description: 'User groups.'
                    health-check:
                        type: str
                        description: 'Health check.'
                    hold-down-time:
                        type: int
                        description: 'Waiting period in seconds when switching from the back-up member to the primary member (0 - 10000000, default = 0).'
                    id:
                        type: int
                        description: 'Priority rule ID (1 - 4000).'
                    internet-service:
                        type: str
                        description: 'Enable/disable use of Internet service for application-based load balancing.'
                        choices:
                            - 'disable'
                            - 'enable'
                    internet-service-ctrl:
                        description: no description
                        type: int
                    internet-service-ctrl-group:
                        type: str
                        description: 'Control-based Internet Service group list.'
                    internet-service-custom:
                        type: str
                        description: 'Custom Internet service name list.'
                    internet-service-custom-group:
                        type: str
                        description: 'Custom Internet Service group list.'
                    internet-service-group:
                        type: str
                        description: 'Internet Service group list.'
                    internet-service-id:
                        type: str
                        description: 'Internet service ID list.'
                    jitter-weight:
                        type: int
                        description: 'Coefficient of jitter in the formula of custom-profile-1.'
                    latency-weight:
                        type: int
                        description: 'Coefficient of latency in the formula of custom-profile-1.'
                    link-cost-factor:
                        type: str
                        description: 'Link cost factor.'
                        choices:
                            - 'latency'
                            - 'jitter'
                            - 'packet-loss'
                            - 'inbandwidth'
                            - 'outbandwidth'
                            - 'bibandwidth'
                            - 'custom-profile-1'
                    link-cost-threshold:
                        type: int
                        description: 'Percentage threshold change of link cost values that will result in policy route regeneration (0 - 10000000, default =...'
                    member:
                        type: str
                        description: 'Member sequence number.'
                    mode:
                        type: str
                        description: 'Control how the priority rule sets the priority of interfaces in the SD-WAN.'
                        choices:
                            - 'auto'
                            - 'manual'
                            - 'priority'
                            - 'sla'
                            - 'load-balance'
                    name:
                        type: str
                        description: 'Priority rule name.'
                    packet-loss-weight:
                        type: int
                        description: 'Coefficient of packet-loss in the formula of custom-profile-1.'
                    priority-members:
                        type: str
                        description: 'Member sequence number list.'
                    protocol:
                        type: int
                        description: 'Protocol number.'
                    quality-link:
                        type: int
                        description: 'Quality grade.'
                    route-tag:
                        type: int
                        description: 'IPv4 route map route-tag.'
                    sla:
                        description: no description
                        type: list
                        suboptions:
                            health-check:
                                type: str
                                description: 'Virtual WAN Link health-check.'
                            id:
                                type: int
                                description: 'SLA ID.'
                    src:
                        type: str
                        description: 'Source address name.'
                    src-negate:
                        type: str
                        description: 'Enable/disable negation of source address match.'
                        choices:
                            - 'disable'
                            - 'enable'
                    src6:
                        type: str
                        description: 'Source address6 name.'
                    start-port:
                        type: int
                        description: 'Start destination port number.'
                    status:
                        type: str
                        description: 'Enable/disable SD-WAN service.'
                        choices:
                            - 'disable'
                            - 'enable'
                    tos:
                        type: str
                        description: 'Type of service bit pattern.'
                    tos-mask:
                        type: str
                        description: 'Type of service evaluated bits.'
                    users:
                        type: str
                        description: 'User name.'
            status:
                type: str
                description: 'Enable/disable SD-WAN.'
                choices:
                    - 'disable'
                    - 'enable'

'''

EXAMPLES = '''
 - hosts: fortimanager-inventory
   collections:
     - fortinet.fortimanager
   connection: httpapi
   vars:
      ansible_httpapi_use_ssl: True
      ansible_httpapi_validate_certs: False
      ansible_httpapi_port: 443
   tasks:
    - name: Configure redundant internet connections using SD-WAN (formerly virtual WAN link).
      fmgr_wanprof_system_virtualwanlink:
         bypass_validation: False
         workspace_locking_adom: <value in [global, custom adom including root]>
         workspace_locking_timeout: 300
         rc_succeeded: [0, -2, -3, ...]
         rc_failed: [-2, -3, ...]
         adom: <your own value>
         wanprof: <your own value>
         wanprof_system_virtualwanlink:
            fail-detect: <value in [disable, enable]>
            health-check:
              -
                  _dynamic-server: <value of string>
                  addr-mode: <value in [ipv4, ipv6]>
                  failtime: <value of integer>
                  http-agent: <value of string>
                  http-get: <value of string>
                  http-match: <value of string>
                  interval: <value of integer>
                  members: <value of string>
                  name: <value of string>
                  packet-size: <value of integer>
                  password: <value of string>
                  port: <value of integer>
                  protocol: <value in [ping, tcp-echo, udp-echo, ...]>
                  recoverytime: <value of integer>
                  security-mode: <value in [none, authentication]>
                  server: <value of string>
                  sla:
                    -
                        id: <value of integer>
                        jitter-threshold: <value of integer>
                        latency-threshold: <value of integer>
                        link-cost-factor:
                          - latency
                          - jitter
                          - packet-loss
                        packetloss-threshold: <value of integer>
                  threshold-alert-jitter: <value of integer>
                  threshold-alert-latency: <value of integer>
                  threshold-alert-packetloss: <value of integer>
                  threshold-warning-jitter: <value of integer>
                  threshold-warning-latency: <value of integer>
                  threshold-warning-packetloss: <value of integer>
                  update-cascade-interface: <value in [disable, enable]>
                  update-static-route: <value in [disable, enable]>
            load-balance-mode: <value in [source-ip-based, weight-based, usage-based, ...]>
            members:
              -
                  _dynamic-member: <value of string>
                  comment: <value of string>
                  gateway: <value of string>
                  gateway6: <value of string>
                  ingress-spillover-threshold: <value of integer>
                  interface: <value of string>
                  priority: <value of integer>
                  seq-num: <value of integer>
                  source: <value of string>
                  source6: <value of string>
                  spillover-threshold: <value of integer>
                  status: <value in [disable, enable]>
                  volume-ratio: <value of integer>
                  weight: <value of integer>
            service:
              -
                  addr-mode: <value in [ipv4, ipv6]>
                  bandwidth-weight: <value of integer>
                  default: <value in [disable, enable]>
                  dscp-forward: <value in [disable, enable]>
                  dscp-forward-tag: <value of string>
                  dscp-reverse: <value in [disable, enable]>
                  dscp-reverse-tag: <value of string>
                  dst: <value of string>
                  dst-negate: <value in [disable, enable]>
                  dst6: <value of string>
                  end-port: <value of integer>
                  gateway: <value in [disable, enable]>
                  groups: <value of string>
                  health-check: <value of string>
                  hold-down-time: <value of integer>
                  id: <value of integer>
                  internet-service: <value in [disable, enable]>
                  internet-service-ctrl: <value of integer>
                  internet-service-ctrl-group: <value of string>
                  internet-service-custom: <value of string>
                  internet-service-custom-group: <value of string>
                  internet-service-group: <value of string>
                  internet-service-id: <value of string>
                  jitter-weight: <value of integer>
                  latency-weight: <value of integer>
                  link-cost-factor: <value in [latency, jitter, packet-loss, ...]>
                  link-cost-threshold: <value of integer>
                  member: <value of string>
                  mode: <value in [auto, manual, priority, ...]>
                  name: <value of string>
                  packet-loss-weight: <value of integer>
                  priority-members: <value of string>
                  protocol: <value of integer>
                  quality-link: <value of integer>
                  route-tag: <value of integer>
                  sla:
                    -
                        health-check: <value of string>
                        id: <value of integer>
                  src: <value of string>
                  src-negate: <value in [disable, enable]>
                  src6: <value of string>
                  start-port: <value of integer>
                  status: <value in [disable, enable]>
                  tos: <value of string>
                  tos-mask: <value of string>
                  users: <value of string>
            status: <value in [disable, enable]>

'''

RETURN = '''
request_url:
    description: The full url requested
    returned: always
    type: str
    sample: /sys/login/user
response_code:
    description: The status of api request
    returned: always
    type: int
    sample: 0
response_message:
    description: The descriptive message of the api response
    type: str
    returned: always
    sample: OK.

'''
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.connection import Connection
from ansible_collections.fortinet.fortimanager.plugins.module_utils.napi import NAPIManager
from ansible_collections.fortinet.fortimanager.plugins.module_utils.napi import check_galaxy_version
from ansible_collections.fortinet.fortimanager.plugins.module_utils.napi import check_parameter_bypass


def main():
    jrpc_urls = [
        '/pm/config/adom/{adom}/wanprof/{wanprof}/system/virtual-wan-link'
    ]

    perobject_jrpc_urls = [
        '/pm/config/adom/{adom}/wanprof/{wanprof}/system/virtual-wan-link/{virtual-wan-link}'
    ]

    url_params = ['adom', 'wanprof']
    module_primary_key = None
    module_arg_spec = {
        'bypass_validation': {
            'type': 'bool',
            'required': False,
            'default': False
        },
        'workspace_locking_adom': {
            'type': 'str',
            'required': False
        },
        'workspace_locking_timeout': {
            'type': 'int',
            'required': False,
            'default': 300
        },
        'rc_succeeded': {
            'required': False,
            'type': 'list'
        },
        'rc_failed': {
            'required': False,
            'type': 'list'
        },
        'adom': {
            'required': True,
            'type': 'str'
        },
        'wanprof': {
            'required': True,
            'type': 'str'
        },
        'wanprof_system_virtualwanlink': {
            'required': False,
            'type': 'dict',
            'options': {
                'fail-detect': {
                    'required': False,
                    'choices': [
                        'disable',
                        'enable'
                    ],
                    'type': 'str'
                },
                'health-check': {
                    'required': False,
                    'type': 'list',
                    'options': {
                        '_dynamic-server': {
                            'required': False,
                            'type': 'str'
                        },
                        'addr-mode': {
                            'required': False,
                            'choices': [
                                'ipv4',
                                'ipv6'
                            ],
                            'type': 'str'
                        },
                        'failtime': {
                            'required': False,
                            'type': 'int'
                        },
                        'http-agent': {
                            'required': False,
                            'type': 'str'
                        },
                        'http-get': {
                            'required': False,
                            'type': 'str'
                        },
                        'http-match': {
                            'required': False,
                            'type': 'str'
                        },
                        'interval': {
                            'required': False,
                            'type': 'int'
                        },
                        'members': {
                            'required': False,
                            'type': 'str'
                        },
                        'name': {
                            'required': False,
                            'type': 'str'
                        },
                        'packet-size': {
                            'required': False,
                            'type': 'int'
                        },
                        'password': {
                            'required': False,
                            'type': 'str'
                        },
                        'port': {
                            'required': False,
                            'type': 'int'
                        },
                        'protocol': {
                            'required': False,
                            'choices': [
                                'ping',
                                'tcp-echo',
                                'udp-echo',
                                'http',
                                'twamp',
                                'ping6'
                            ],
                            'type': 'str'
                        },
                        'recoverytime': {
                            'required': False,
                            'type': 'int'
                        },
                        'security-mode': {
                            'required': False,
                            'choices': [
                                'none',
                                'authentication'
                            ],
                            'type': 'str'
                        },
                        'server': {
                            'required': False,
                            'type': 'str'
                        },
                        'sla': {
                            'required': False,
                            'type': 'list',
                            'options': {
                                'id': {
                                    'required': False,
                                    'type': 'int'
                                },
                                'jitter-threshold': {
                                    'required': False,
                                    'type': 'int'
                                },
                                'latency-threshold': {
                                    'required': False,
                                    'type': 'int'
                                },
                                'link-cost-factor': {
                                    'required': False,
                                    'type': 'list',
                                    'choices': [
                                        'latency',
                                        'jitter',
                                        'packet-loss'
                                    ]
                                },
                                'packetloss-threshold': {
                                    'required': False,
                                    'type': 'int'
                                }
                            }
                        },
                        'threshold-alert-jitter': {
                            'required': False,
                            'type': 'int'
                        },
                        'threshold-alert-latency': {
                            'required': False,
                            'type': 'int'
                        },
                        'threshold-alert-packetloss': {
                            'required': False,
                            'type': 'int'
                        },
                        'threshold-warning-jitter': {
                            'required': False,
                            'type': 'int'
                        },
                        'threshold-warning-latency': {
                            'required': False,
                            'type': 'int'
                        },
                        'threshold-warning-packetloss': {
                            'required': False,
                            'type': 'int'
                        },
                        'update-cascade-interface': {
                            'required': False,
                            'choices': [
                                'disable',
                                'enable'
                            ],
                            'type': 'str'
                        },
                        'update-static-route': {
                            'required': False,
                            'choices': [
                                'disable',
                                'enable'
                            ],
                            'type': 'str'
                        }
                    }
                },
                'load-balance-mode': {
                    'required': False,
                    'choices': [
                        'source-ip-based',
                        'weight-based',
                        'usage-based',
                        'source-dest-ip-based',
                        'measured-volume-based'
                    ],
                    'type': 'str'
                },
                'members': {
                    'required': False,
                    'type': 'list',
                    'options': {
                        '_dynamic-member': {
                            'required': False,
                            'type': 'str'
                        },
                        'comment': {
                            'required': False,
                            'type': 'str'
                        },
                        'gateway': {
                            'required': False,
                            'type': 'str'
                        },
                        'gateway6': {
                            'required': False,
                            'type': 'str'
                        },
                        'ingress-spillover-threshold': {
                            'required': False,
                            'type': 'int'
                        },
                        'interface': {
                            'required': False,
                            'type': 'str'
                        },
                        'priority': {
                            'required': False,
                            'type': 'int'
                        },
                        'seq-num': {
                            'required': False,
                            'type': 'int'
                        },
                        'source': {
                            'required': False,
                            'type': 'str'
                        },
                        'source6': {
                            'required': False,
                            'type': 'str'
                        },
                        'spillover-threshold': {
                            'required': False,
                            'type': 'int'
                        },
                        'status': {
                            'required': False,
                            'choices': [
                                'disable',
                                'enable'
                            ],
                            'type': 'str'
                        },
                        'volume-ratio': {
                            'required': False,
                            'type': 'int'
                        },
                        'weight': {
                            'required': False,
                            'type': 'int'
                        }
                    }
                },
                'service': {
                    'required': False,
                    'type': 'list',
                    'options': {
                        'addr-mode': {
                            'required': False,
                            'choices': [
                                'ipv4',
                                'ipv6'
                            ],
                            'type': 'str'
                        },
                        'bandwidth-weight': {
                            'required': False,
                            'type': 'int'
                        },
                        'default': {
                            'required': False,
                            'choices': [
                                'disable',
                                'enable'
                            ],
                            'type': 'str'
                        },
                        'dscp-forward': {
                            'required': False,
                            'choices': [
                                'disable',
                                'enable'
                            ],
                            'type': 'str'
                        },
                        'dscp-forward-tag': {
                            'required': False,
                            'type': 'str'
                        },
                        'dscp-reverse': {
                            'required': False,
                            'choices': [
                                'disable',
                                'enable'
                            ],
                            'type': 'str'
                        },
                        'dscp-reverse-tag': {
                            'required': False,
                            'type': 'str'
                        },
                        'dst': {
                            'required': False,
                            'type': 'str'
                        },
                        'dst-negate': {
                            'required': False,
                            'choices': [
                                'disable',
                                'enable'
                            ],
                            'type': 'str'
                        },
                        'dst6': {
                            'required': False,
                            'type': 'str'
                        },
                        'end-port': {
                            'required': False,
                            'type': 'int'
                        },
                        'gateway': {
                            'required': False,
                            'choices': [
                                'disable',
                                'enable'
                            ],
                            'type': 'str'
                        },
                        'groups': {
                            'required': False,
                            'type': 'str'
                        },
                        'health-check': {
                            'required': False,
                            'type': 'str'
                        },
                        'hold-down-time': {
                            'required': False,
                            'type': 'int'
                        },
                        'id': {
                            'required': False,
                            'type': 'int'
                        },
                        'internet-service': {
                            'required': False,
                            'choices': [
                                'disable',
                                'enable'
                            ],
                            'type': 'str'
                        },
                        'internet-service-ctrl': {
                            'required': False,
                            'type': 'int'
                        },
                        'internet-service-ctrl-group': {
                            'required': False,
                            'type': 'str'
                        },
                        'internet-service-custom': {
                            'required': False,
                            'type': 'str'
                        },
                        'internet-service-custom-group': {
                            'required': False,
                            'type': 'str'
                        },
                        'internet-service-group': {
                            'required': False,
                            'type': 'str'
                        },
                        'internet-service-id': {
                            'required': False,
                            'type': 'str'
                        },
                        'jitter-weight': {
                            'required': False,
                            'type': 'int'
                        },
                        'latency-weight': {
                            'required': False,
                            'type': 'int'
                        },
                        'link-cost-factor': {
                            'required': False,
                            'choices': [
                                'latency',
                                'jitter',
                                'packet-loss',
                                'inbandwidth',
                                'outbandwidth',
                                'bibandwidth',
                                'custom-profile-1'
                            ],
                            'type': 'str'
                        },
                        'link-cost-threshold': {
                            'required': False,
                            'type': 'int'
                        },
                        'member': {
                            'required': False,
                            'type': 'str'
                        },
                        'mode': {
                            'required': False,
                            'choices': [
                                'auto',
                                'manual',
                                'priority',
                                'sla',
                                'load-balance'
                            ],
                            'type': 'str'
                        },
                        'name': {
                            'required': False,
                            'type': 'str'
                        },
                        'packet-loss-weight': {
                            'required': False,
                            'type': 'int'
                        },
                        'priority-members': {
                            'required': False,
                            'type': 'str'
                        },
                        'protocol': {
                            'required': False,
                            'type': 'int'
                        },
                        'quality-link': {
                            'required': False,
                            'type': 'int'
                        },
                        'route-tag': {
                            'required': False,
                            'type': 'int'
                        },
                        'sla': {
                            'required': False,
                            'type': 'list',
                            'options': {
                                'health-check': {
                                    'required': False,
                                    'type': 'str'
                                },
                                'id': {
                                    'required': False,
                                    'type': 'int'
                                }
                            }
                        },
                        'src': {
                            'required': False,
                            'type': 'str'
                        },
                        'src-negate': {
                            'required': False,
                            'choices': [
                                'disable',
                                'enable'
                            ],
                            'type': 'str'
                        },
                        'src6': {
                            'required': False,
                            'type': 'str'
                        },
                        'start-port': {
                            'required': False,
                            'type': 'int'
                        },
                        'status': {
                            'required': False,
                            'choices': [
                                'disable',
                                'enable'
                            ],
                            'type': 'str'
                        },
                        'tos': {
                            'required': False,
                            'type': 'str'
                        },
                        'tos-mask': {
                            'required': False,
                            'type': 'str'
                        },
                        'users': {
                            'required': False,
                            'type': 'str'
                        }
                    }
                },
                'status': {
                    'required': False,
                    'choices': [
                        'disable',
                        'enable'
                    ],
                    'type': 'str'
                }
            }

        }
    }

    params_validation_blob = []
    check_galaxy_version(module_arg_spec)
    module = AnsibleModule(argument_spec=check_parameter_bypass(module_arg_spec, 'wanprof_system_virtualwanlink'),
                           supports_check_mode=False)

    fmgr = None
    if module._socket_path:
        connection = Connection(module._socket_path)
        fmgr = NAPIManager(jrpc_urls, perobject_jrpc_urls, module_primary_key, url_params, module, connection, top_level_schema_name='data')
        fmgr.validate_parameters(params_validation_blob)
        fmgr.process_partial_curd()
    else:
        module.fail_json(msg='MUST RUN IN HTTPAPI MODE')
    module.exit_json(meta=module.params)


if __name__ == '__main__':
    main()
